// nl3wusertype.cpp : DLL AvP[VpɃGNX|[g֐`܂B


#include "stdafx.h"
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include "dlliftype.h"


struct PinSigname{
	const char* pszPinNum;		//Pin number
	const char* pszNetName;		//Net name
};


int comparer( const void* elem1, const void*  elem2 )
{
	const char* pszelem1number = ((PinSigname*)elem1)->pszPinNum;
	const char* pszelem2number = ((PinSigname*)elem2)->pszPinNum;

	if(pszelem1number == NULL){
		pszelem1number = "";
	}

	if(pszelem2number == NULL){
		pszelem2number = "";
	}

	int num1 = atoi(pszelem1number);
	int num2 = atoi(pszelem2number);

	if(num1!=num2) return num1-num2;

	return strcmp( pszelem1number, pszelem2number);
}


// 0: no pin
// 1 or larger: pincount
// -1: too many pins
int pickupPin(const char* refnum, const SExportNetlistInfo* pNetInfo, PinSigname *arrayPinSigname, int maxpincount)
{
	int pincount = 0;

	for(int i=0;i<pNetInfo->nNetValueCount;i++){
		//loop for scanning pins in current net
		for(int j=0;j< pNetInfo->array_NetValue[i].nPinCount; j++){
			if(strcmp(refnum, pNetInfo->array_NetValue[i].pArrayPinValue[j].pszRefNum)==0){
				arrayPinSigname[pincount].pszNetName = pNetInfo->array_NetValue[i].pszNetName;
				arrayPinSigname[pincount].pszPinNum = pNetInfo->array_NetValue[i].pArrayPinValue[j].pszPinNum;
				pincount++;
				if(pincount >=  maxpincount)return -1;
			}
		}
	}
	if(pincount==0) return 0;
	qsort(arrayPinSigname,pincount,sizeof(PinSigname),comparer);
	return pincount;
}


void __stdcall typeName(char* buff,int buffsize)
{
	static const char* fmtname = "KiCAD";	//  This string is displayed in NL3W as a netlist type.
	strncpy(buff,fmtname,buffsize-1);
	buff[buffsize-1] = '\0';
}


#define MAX_PINCOUNT 2048
#define REFNUM_BUFFSIZE 128

/////////////////////////////////////////////////////////////////////////////////////
//ERROR CODE
//successful: 0
//file error: -1
//no net data: -2
//user define error: =>1
int __stdcall fnWriteNetList(const char* filename, const SExportNetlistInfo* pNetInfo)
{
	PinSigname arrayPinSigname[MAX_PINCOUNT];
	char refNumber[REFNUM_BUFFSIZE];
	
	if(filename == NULL) return -1;
	FILE* fp = fopen(filename,"wt");
	if(fp==NULL) return -1;

	if(pNetInfo==NULL) return -2;


	SYSTEMTIME localTime;
	GetLocalTime(&localTime);

	unsigned timestamp = (unsigned)(::time(NULL)) - (pNetInfo->nComponentInfoCount+100);

	fprintf(fp,"# EESchema Netlist Version 1.1 created %4d/%02d/%02d %02d:%02d:%02d\n",
		localTime.wYear,
		localTime.wMonth,
		localTime.wDay,
		localTime.wHour,
		localTime.wMinute,
		localTime.wSecond
	);

	fprintf(fp,"(\n");


	for(int i=0;i<pNetInfo->nComponentInfoCount;i++){

		sprintf_s(refNumber, REFNUM_BUFFSIZE, "%s%d", 
			pNetInfo->array_CompInfo[i].pszPrefix,
			pNetInfo->array_CompInfo[i].nSuffix);

		const char* pcszPackage;
		if(pNetInfo->array_CompInfo[i].pszPackage != NULL && strlen(pNetInfo->array_CompInfo[i].pszPackage)>0){
			pcszPackage = pNetInfo->array_CompInfo[i].pszPackage;
		}else{
			pcszPackage = "$noname";
		}
		fprintf(fp," ( /%08X %s %s %s\n", timestamp, pcszPackage, refNumber, pNetInfo->array_CompInfo[i].pszName);
		int pincount = pickupPin(refNumber,pNetInfo, arrayPinSigname, MAX_PINCOUNT);
		if(pincount==-1){
			fprintf(fp,"***** ERROR: %s has too many pins. (over %d pins) *****\n",refNumber,MAX_PINCOUNT);
			fclose(fp);
			return 1;
		}
		for(int j=0;j<pincount;j++){
			fprintf(fp,"   (  %s %s  )\n", arrayPinSigname[j].pszPinNum, arrayPinSigname[j].pszNetName);
		}
		fprintf(fp," )\n");
	}
	fprintf(fp,")\n");
	fprintf(fp,"*\n");

	fclose(fp);
	return 0;
}



